/*
    $File$
    Date        Author          Changes
    Nov 21 2000 Richard Jones   Created
 */

 
package MetaTF.Parser;

import java.util.*;
/**
 * Abstract field (part of the symbol table)
 *    children are tagField, NamedField, RepeatField, TupleField
 *
 * @see <a href="metatf.jjt">The MetaTF Grammar</a>
 * @see TagField
 * @see NamedField
 * @see RepeatField
 * @see TupleField
 * @version rel-1-2 ($Revision: 1.4 $, $Date: 2001/03/29 15:53:18 $),
 * @author Richard Jones
**/
public abstract class Field implements HasFields {
  private static final String cvsheader = "$Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/Parser/Field.java,v 1.4 2001/03/29 15:53:18 rej Exp $";

    /** The line in the DTD where this field was defined **/
    protected int beginLine;

    Field() {}
    
    /**
     * Does this field contain a named subfield?
     *
     * @param s the name of the subfield
     * @return whether this field contains a subfield named s
    **/
    public abstract boolean containsField(String s);

    /**
     * Add a subfield to this field
     *
     * @param f the subfield
    **/
    public abstract void addField(Field f); // public only because it's an interface method :-(

    /**
     * Do a macro replacement of any fields that do not have
     * properties defined locally.
     *
     * @param inheritedRecords0 Map <name, Record> of records inherited from section 0
     * @param inheritedRecords Map <name, Record> of records inherited from this section
     * @param inheritedFields Map <name, Field> of inherited fields
     * @return the modified field
    **/
    abstract Field replaceFields(Map inheritedRecords0, Map inheritedRecords, Map inheritedFields);

    /**
     * Check that this field is well formed
     * i.e. has a type and an encoding.
     * All macro substition must be done before checkFields is called
     *
     * @exception DTDException IF this field is not well-formed
     * @see #replaceFields
     */
    abstract void checkFields() throws DTDException;

    /**
     * Add a property to this field
     *
     * @param type the type
     * @param field the name of the field. The property is only added if the field names match
     * @param property the property
     * @param value the value of the property
     * @param beginLine the line in the DTD at which this property was defined
     * @exception DTDException
    **/
    abstract boolean addProperty(String type, String field, String property, Value value, int beginLine)
            throws DTDException;

    /** @return the number of named subfields of this field **/
    public abstract int getNumNamedFields();

    /**
     * Get the tag of this field
     *
     * @return the tag, if any
     * @exception NoTagException IF this field does not contain a tag
    **/
    public abstract int getTag() throws NoTagException;

    /**
     * Is this field tagged?
     *
     * @return whether the field is tagged
    **/
    public abstract boolean isTagged();

    /** @return the name of the field **/
    public abstract String getName();

    /** @return the type of the field **/
    public abstract String getType();

    /**
     * Dump this field
     *
     * @return a prettified representation of this field
    **/
    public abstract String dump(String prefix);

    /** Accept the visitor **/
    public abstract Object accept(DTDVisitor visitor, Object data);
}
