/*
    $File$
    Date        Author          Changes
    Nov 21 2000 Richard Jones   Created

    Copyright (c) 2000, Richard Jones
    All rights reserved.
 */

 
package MetaTF.Parser;

import java.util.*;
import MetaTF.Panic;

/**
 * Repeat field
 *
 * @see <a href="metatf.jjt">The MetaTF Grammar</a>
 * @see MetaTF.Parser.Field
 * @version rel-1-2 ($Revision: 1.3 $, $Date: 2000/12/15 16:29:55 $),
 * @author Richard Jones
**/
public class RepeatField extends Field {
   private static final String cvsheader = "$Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/Parser/RepeatField.java,v 1.3 2000/12/15 16:29:55 rej Exp $";

    /** The name of the repeat **/
    private NamedField repeat;

    /** The field to be repeated **/
    private Field repeatable;
    
    RepeatField() {
        repeat = null;
        repeatable = null;
    }

    /**
     * Add a field.
     * This field is the repeat (if not set), in which case it must be a NamedField
     * else it is the field to be repeated (if not set).
     * It is an error if both fields already assigned.
     * public only because it is an interface method
     *
     * @param f the field to be added
    **/
    public void addField(Field f) {
        if(repeat==null) {
            if(f instanceof NamedField)
                repeat = (NamedField)f;
             else
                 Panic.panic("RepeatField.addField gets bad field for repeat");
        } else if(repeatable==null)
            repeatable = f;
        else
            Panic.panic("RepeatField.addField finds both fields already populated");
    }

    void addField(TupleField f) {
        repeatable = f;
    }

    /** @return the repeat field **/
    public NamedField getRepeat() { return repeat; }

    /** @return the repeated field **/
    public Field getRepeatable() { return repeatable; }

    public Field replaceFields(Map inheritedRecords0, Map inheritedRecords, Map inheritedFields) {
        // doesn't make sense to replace the repeat field with a record
        repeat = (NamedField)repeat.replaceFields(null, null, inheritedFields);
    	repeatable = repeatable.replaceFields(inheritedRecords0, inheritedRecords, inheritedFields);
    	return this;
    }

    public void checkFields() throws DTDException {
        repeat.checkFields();
        repeatable.checkFields();
    }

    public boolean containsField(String s) {
        return repeatable.containsField(s);
    }

    public boolean addProperty(String type, String field, String property, Value value, int l) throws DTDException {
    	boolean result = repeat.addProperty(type, field, property, value, l);
        return repeatable.addProperty(type, field, property, value, l) || result;
    }

    public int getNumNamedFields() {
        return repeatable.getNumNamedFields()+1;
    }
     
    /**
     * Get the tag of this field
     *
     * @exception NoTagException ALWAYS
    **/
    public int getTag() throws NoTagException {
        throw new NoTagException("l. "+beginLine+": repeat field");
    }

    public boolean isTagged() { return false; }

    public String getName() { return ""; }

    public String getType() { return null; }
    
    public String dump(String prefix) {
        return repeat.dump(prefix)+"\n"+repeatable.dump(prefix+"* ");
    }

    public Object accept(DTDVisitor visitor, Object data) {
        return visitor.visit(this, data);
    }
}
