/* Generated By:JavaCC: Do not edit this line. TokenMgrError.java Version 0.7pre2 */
/**
 *
 * Copyright (C) 2000 Richard Jones.
 * All rights reserved
 *
 * Grammar for MetaTF 1.2
 *
 * Based on Grammar for Meta-TF 1.0, for which details can be found in
 * "Designing a Trace Format for Heap Allocation Events",
 * Trishul Chilimbi, Richard Jones and Benjamin Zorn,
 * ISMM 2000, Minneapolis, October 2000, ACM.
 *
 * Changes since MetaTF 1.0:
 *
 * MetaTF 1.2
 * 1. Record fields are typed. The primitive types are:
 *     byte, short, int, long, string or index.
 * 2. Only the following interpretations are recognised:
 *    None, Delta, Default, BaseOffset and Stride.
 *    Strings may only have interpretations None or Default.
 * 3. The syntax for defining fields is changed. For example,
 *        int foo = Default 3;
 * 4. Only single line comments starting with a # are allowed.
 *
 * MetaTF 1.1
 * 1. Removed RHS of property in the form "field.property"
 * 2. Removed anonymous tuples, e.g. rec : (a, b, (x, y), c)
 *    These must now be written as either
 *	rec : (a, b, x, y, z)
 *    or
 *	xy : (x, y)
 *	rec: (a, b, xy, c)
 * 3. In repeated events, such as a * b, clarify that
 *    a is a new field, not a reference to an existing one.
 */

// $Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/Parser/TokenMgrError.java,v 1.3 2000/12/15 16:29:55 rej Exp $

package MetaTF.Parser;

public class TokenMgrError extends Error
{
   /*
    * Ordinals for various reasons why an Error of this type can be thrown.
    */

   /**
    * Lexical error occured.
    */
   static final int LEXICAL_ERROR = 0;

   /**
    * An attempt wass made to create a second instance of a static token manager.
    */
   static final int STATIC_LEXER_ERROR = 1;

   /**
    * Tried to change to an invalid lexical state.
    */
   static final int INVALID_LEXICAL_STATE = 2;

   /**
    * Detected (and bailed out of) an infinite loop in the token manager.
    */
   static final int LOOP_DETECTED = 3;

   /**
    * Indicates the reason why the exception is thrown. It will have
    * one of the above 4 values.
    */
   int errorCode;

   /**
    * Replaces unprintable characters by their espaced (or unicode escaped)
    * equivalents in the given string
    */
   protected static final String addEscapes(String str) {
      StringBuffer retval = new StringBuffer();
      char ch;
      for (int i = 0; i < str.length(); i++) {
        switch (str.charAt(i))
        {
           case 0 :
              continue;
           case '\b':
              retval.append("\\b");
              continue;
           case '\t':
              retval.append("\\t");
              continue;
           case '\n':
              retval.append("\\n");
              continue;
           case '\f':
              retval.append("\\f");
              continue;
           case '\r':
              retval.append("\\r");
              continue;
           case '\"':
              retval.append("\\\"");
              continue;
           case '\'':
              retval.append("\\\'");
              continue;
           case '\\':
              retval.append("\\\\");
              continue;
           default:
              if ((ch = str.charAt(i)) < 0x20 || ch > 0x7e) {
                 String s = "0000" + Integer.toString(ch, 16);
                 retval.append("\\u" + s.substring(s.length() - 4, s.length()));
              } else {
                 retval.append(ch);
              }
              continue;
        }
      }
      return retval.toString();
   }

   /**
    * Returns a detailed message for the Error when it is thrown by the
    * token manager to indicate a lexical error.
    * Parameters : 
    *    EOFSeen     : indicates if EOF caused the lexicl error
    *    curLexState : lexical state in which this error occured
    *    errorLine   : line number when the error occured
    *    errorColumn : column number when the error occured
    *    errorAfter  : prefix that was seen before this error occured
    *    curchar     : the offending character
    * Note: You can customize the lexical error message by modifying this method.
    */
   private static final String LexicalError(boolean EOFSeen, int lexState, int errorLine, int errorColumn, String errorAfter, char curChar) {
      return("Lexical error at line " +
           errorLine + ", column " +
           errorColumn + ".  Encountered: " +
           (EOFSeen ? "<EOF> " : ("\"" + addEscapes(String.valueOf(curChar)) + "\"") + " (" + (int)curChar + "), ") +
           "after : \"" + addEscapes(errorAfter) + "\"");
   }

   /**
    * You can also modify the body of this method to customize your error messages.
    * For example, cases like LOOP_DETECTED and INVALID_LEXICAL_STATE are not
    * of end-users concern, so you can return something like : 
    *
    *     "Internal Error : Please file a bug report .... "
    *
    * from this method for such cases in the release version of your parser.
    */
   public String getMessage() {
      return super.getMessage();
   }

   /*
    * Constructors of various flavors follow.
    */

   public TokenMgrError() {
   }

   public TokenMgrError(String message, int reason) {
      super(message);
      errorCode = reason;
   }

   public TokenMgrError(boolean EOFSeen, int lexState, int errorLine, int errorColumn, String errorAfter, char curChar, int reason) {
      this(LexicalError(EOFSeen, lexState, errorLine, errorColumn, errorAfter, curChar), reason);
   }
}
