/*
    $File$
    Date        Author          Changes
    Nov 21 2000 Richard Jones   Created

    Copyright (c) 2000, Richard Jones
    All rights reserved.
 */

 
package MetaTF.Parser;

import MetaTF.Panic;
import java.util.*;

/**
 * Tuple field
 *
 * @see <a href="metatf.jjt">The MetaTF Grammar</a>
 * @see MetaTF.Parser.Field
 * @version rel-1-2 ($Revision: 1.3 $, $Date: 2000/12/15 16:29:55 $),
 * @author Richard Jones
**/

public class TupleField extends Field {
  private static final String cvsheader = "$Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/Parser/TupleField.java,v 1.3 2000/12/15 16:29:55 rej Exp $";

    /** Tuple's type, i.e. the name of the record **/
    private String name;

    /** The components of the tuple : List <Field> **/
    private List fields;
    
    /**
     * Allocate a new object representing a tuple
     *
     * @param s the name of the tuple
     * @param fs the list of components of this tuple
    **/
    TupleField(String s, List fs) {
    	name = s;
        fields = fs.isEmpty() ?
		     new ArrayList() :
		     fs;
    }

    /** Allocate a new object representing a tuple **/
    TupleField() {
        this("",new ArrayList());
    }

    /**
     * Allocate a new object representing a tuple
     *
     * @param name the name of the tuple
    **/
    TupleField(String name) {
        this(name,new ArrayList());
    }

    // public only because it is an interface field
    public void addField(Field f) {
        fields.add(f);
    }

    /** @return a list of the top level components of this tuple **/
    public List getFields() { return fields; }

    public String getName() { return (name==null) ? "" : name; }

    Field replaceFields(Map inheritedRecords0, Map inheritedRecords, Map inheritedFields) {
    	for(int i=0; i<fields.size(); i++) {
    	    Field f = (Field)fields.get(i);
    	    if(f!=null) {
    	        fields.set(i, f.replaceFields(inheritedRecords0, inheritedRecords, inheritedFields));
    	    } else
    	        Panic.panic("TupleField"+name+".replaceFields finds null field "+i+"!!");
        }
        return this;
    }

    void checkFields() throws DTDException {
        Iterator it = fields.iterator();
        while(it.hasNext()) {
            Field f = (Field)it.next();
            f.checkFields();
        }
    }

    public boolean containsField(String s) {
        Iterator it = fields.iterator();
        while(it.hasNext()) {
            Field f = (Field)it.next();
            if(f.containsField(s))
                return true;
        }
        return false;
    }

    public boolean addProperty(String type, String field, String property, Value value, int l) 
            throws DTDException {
        boolean result = false;
        Iterator it = fields.iterator();
        while(it.hasNext()) {
            Field f = (Field)it.next();
            result = f.addProperty(type, field, property, value, l) || result;
        }        
        return result;
    }

    public int getNumNamedFields() {
        int no = 0;
        if(fields!=null) {
            Iterator it = fields.iterator();
            while(it.hasNext()) {
                Field f = (Field)it.next();
                no += f.getNumNamedFields();
            }
        }
        return no;
    }

    public int getTag() throws NoTagException {
        return ((Field)fields.get(0)).getTag();
    }

    public boolean isTagged() { return ((Field)fields.get(0)).isTagged(); }

    public String getType() { return null; }
    
    public String dump(String prefix) {
        String result = prefix+"(";
        if(name!=null)
            result = result+name+":";
    	Iterator it = fields.iterator();
    	while(it.hasNext()) {
     	   Field f = (Field)it.next();
     	   result = result+"\n"+f.dump(prefix+"  ");
        }
     	return result+"\n"+prefix+")";
     }

    public Object accept(DTDVisitor visitor, Object data) {
        return visitor.visit(this, data);
    }
}
