/*
    $File$
    Date        Author          Changes
    Nov 21 2000 Richard Jones   Created
    
    Copyright (c) 2000, Richard Jones
    All rights reserved.
 */
 
package MetaTF.Runtime;

import java.io.IOException;

/**
 * LowLevelWriter interface
 *
 * @version rel-1-2 ($Revision: 1.1 $, $Date: 2000/12/15 16:35:32 $),
 * @author Richard Jones
**/
public interface LowLevelWriter {

    /**
     * Write a long
     *
     * @exception IOException IF write fails
    **/
    public void write(long value) throws IOException;

    /**
     * Write an int
     *
     * @exception IOException IF write fails
    **/
    public void write(int value) throws IOException;

    /**
     * Write a string, adding quotes
     *
     * @exception IOException IF write fails
    **/
    public void write(String value) throws IOException;

    /**
     * Write a char, such as special tags, etc.
     *
     * @exception IOException IF write fails
    **/
    public void write(char value) throws IOException; 

    /**
     * Write a new line. This is called to terminate the record
     *
     * @exception IOException IF write fails
    **/
    public void writeln() throws IOException;

    /**
     * Write a plain string (no quotes), terminated by a new line
     * Only used to write comment records
     *
     * @exception IOException IF write fails
    **/
    public void writeln(String s) throws IOException;

    /**
     * Flush the output stream
    **/
    public void flush();

    /**
     * Close the output stream
    **/
    public void close();
}
